#pragma once

const int BTT_API_VERSION = 2506;

#pragma pack(push)
#pragma pack(1)
typedef struct _BTT_PREFILE_DATA{
  const int                            _filename_size_in_bytes;
  const int                            _r1;
        int                            _file_flags;
        short                          _codesize;
        short                          _decode_engine;
        __int64                        _base_address;
  const void                          *_r2[4];
  const wchar_t                        _filename[1];
  }BTT_PREFILE_DATA;

typedef struct _BTT_FILE_DATA{
  const int                            _filename_size_in_bytes;
  const int                            _file_index;
  const int                            _file_flags;
  const short                          _codesize;
  const short                          _decode_engine;
  const __int64                        _base_address;
  const void                          *_r2[4];
        void                          *_p_second_parser_data;
        void                          *_p_main_parser_data;
  const wchar_t                        _filename[1];
  }BTT_FILE_DATA;
#pragma pack()

enum{ // BttFlags()
  btt_second_parser                   = 0x0002,
};

enum{ // _file_flags
  file_parser_is_reenterable          = 0x00000010,
  file_dialog_data_enable             = 0x00000100,
  file_dialog_code16_enable           = 0x00000200,
  file_dialog_code32_enable           = 0x00000400,
  file_dialog_code64_enable           = 0x00000800,
  file_dialog_address_enable          = 0x00001000,
  file_dialog_data_little_endian      = 0x00002000,
  file_dialog_data_big_endian         = 0x00004000,
  file_dialog_scan_refers             = 0x00008000,
  file_dialog_scan_wide_odd           = 0x00100000,
  file_dialog_scan_wide_even          = 0x00200000,
  file_dialog_scan_ascii              = 0x00400000,
  file_dialog_scan_latin_only         = 0x00800000,
  file_dialog_scan_ascii_latin_only   = (file_dialog_scan_ascii|file_dialog_scan_latin_only),
};

enum decode_engine_t{
  decode_engine_none                  = 0,
  decode_engine_ia32                  = 1,
};

enum bookmark_color_t{
  bookmark_red                        = 0,
  bookmark_blue,
  bookmark_green,
  bookmark_yellow,
};

enum string_ascii_cp_t{
  ascii_cp_ansi                       = 0,
  ascii_cp_oem,
  ascii_cp_utf8,
};

#define BTT_EXPORT extern "C" __declspec(dllexport)
#ifndef BTT_API
#define BTT_API    extern "C" __declspec(dllimport)
#endif

//  necessarily
BTT_EXPORT int                  ApiVersionUsed();
BTT_EXPORT bool                 IKnowThisFile(            BTT_PREFILE_DATA *p_btt_prefile_data );
BTT_EXPORT bool                 ParseFile(                BTT_FILE_DATA    *p_btt_file_data );

BTT_EXPORT unsigned long        BttID();  // above 0xFFFF
BTT_EXPORT const wchar_t       *BttTitle();
//BTT_EXPORT unsigned long        BttFlags();
//BTT_EXPORT void                 OnFileOpen(                BTT_FILE_DATA    *p_btt_file_data );
//BTT_EXPORT void                 OnFileClose(               BTT_FILE_DATA    *p_btt_file_data );
//BTT_EXPORT __int64              ToAddress(           const BTT_FILE_DATA    *p_btt_file_data, __int64 qw );
//BTT_EXPORT int                  VirtualRead(         const BTT_FILE_DATA    *p_btt_file_data, __int64 address, unsigned char *p, int nbytes );

//BTT_API    bool                 BttGetSaverDir(            wchar_t *p_saver_dir, int saver_dir_max );

//BTT_API    bool                 BttIsAbort(                int file_index );
BTT_API    bool                 BttFileLog(                int file_index, const wchar_t *plogline );
//BTT_API    bool                 BttFileParseStage(         int file_index, const wchar_t *pstage_text );
//BTT_API    bool                 BttFileGoAddress(          int file_index, __int64 address );

BTT_API    bool                 BttFileAddCodeSegment(     int file_index, __int64 address, __int64 foffset, __int64 fsize, __int64 vsize );
BTT_API    bool                 BttFileAddDataSegment(     int file_index, __int64 address, __int64 foffset, __int64 fsize, __int64 vsize );
BTT_API    bool                 BttFileSegmentName(        int file_index, __int64 address, const wchar_t *psection_name );
//BTT_API    bool                 BttFileSegmentCollapse(    int file_index, __int64 address, bool bCollapsed );

BTT_API    bool                 BttFileMakeCollapse(       int file_index, __int64 address, unsigned long size, bool bCollapsed );

//BTT_API    bool                 BttFileAddBookmark(        int file_index, __int64 address, const wchar_t *pnote, bookmark_color_t bookmark_color );

//BTT_API    bool                 BttFileAddName(            int file_index, __int64 address, const wchar_t *pname, const wchar_t literal_group = L' ' );

BTT_API    bool                 BttFileMakeDec(            int file_index );
BTT_API    bool                 BttFileMakeHex(            int file_index );
//BTT_API    bool                 BttFileMakeSign(           int file_index );
//BTT_API    bool                 BttFileMakeUnsign(         int file_index );
//BTT_API    bool                 BttFileDataLittleEndian(   int file_index );
//BTT_API    bool                 BttFileDataBigEndian(      int file_index );
BTT_API    bool                 BttFileMakePerline(        int file_index, int nPerline );
//BTT_API    bool                 BttFileMakeFloatFormat(    int file_index, wchar_t *pformat );

BTT_API    bool                 BttFileSetRunAddress(      int file_index, __int64 address );
//BTT_API    __int64              BttFileGetRunAddress(      int file_index );

BTT_API    bool                 BttFileCommentline(        int file_index, __int64 address, const wchar_t *pcomment );
//BTT_API    bool                 BttFileCommentOnline(      int file_index, __int64 address, const wchar_t *pcomment );
BTT_API    bool                 BttFileRunCommentline(     int file_index, const wchar_t *pcomment );
BTT_API    bool                 BttFileRunCommentOnline(   int file_index, const wchar_t *pcomment );

BTT_API    bool                 BttFileMakeCode(           int file_index, __int64 address );
BTT_API    bool                 BttFileMakeCodes(          int file_index, __int64 address, unsigned long size );
//BTT_API    bool                 BttFileRunCode(            int file_index );
//BTT_API    bool                 BttFileRunCodes(           int file_index, unsigned long size );

BTT_API    bool                 BttFileMakeDump(           int file_index, __int64 address, unsigned long size, bool bCollapsed );
BTT_API    bool                 BttFileRunDump(            int file_index, unsigned long size, bool bCollapsed );

//BTT_API    bool                 BttFileMakeByteline(       int file_index, __int64 address, unsigned long one_llength );
//BTT_API    bool                 BttFileMakeBytelines(      int file_index, __int64 address, unsigned long one_llength, int nlines, bool bCollapsed );
//BTT_API    bool                 BttFileRunByteline(        int file_index, unsigned long one_llength );
//BTT_API    bool                 BttFileRunBytelines(       int file_index, unsigned long one_llength, int nlines, bool bCollapsed );

//BTT_API    bool                 BttFileMakeByte(           int file_index, __int64 address );
//BTT_API    bool                 BttFileMakeBytes(          int file_index, __int64 address, int nBytes );
//BTT_API    bool                 BttFileRunByte(            int file_index );
//BTT_API    bool                 BttFileRunBytes(           int file_index, int nBytes );
//BTT_API    bool                 BttFileMakeWord(           int file_index, __int64 address );
BTT_API    bool                 BttFileMakeWords(          int file_index, __int64 address, int nWords );
BTT_API    bool                 BttFileRunWord(            int file_index );
BTT_API    bool                 BttFileRunWords(           int file_index, int nWords );
//BTT_API    bool                 BttFileMakeDword(          int file_index, __int64 address );
//BTT_API    bool                 BttFileMakeDwords(         int file_index, __int64 address, int nDwords );
//BTT_API    bool                 BttFileRunDword(           int file_index );
//BTT_API    bool                 BttFileRunDwords(          int file_index, int nDwords );
//BTT_API    bool                 BttFileMakeQword(          int file_index, __int64 address );
//BTT_API    bool                 BttFileMakeQwords(         int file_index, __int64 address, int nQwords );
//BTT_API    bool                 BttFileRunQword(           int file_index );
//BTT_API    bool                 BttFileRunQwords(          int file_index, int nQwords );

//BTT_API    bool                 BttFileMakeFloat(          int file_index, __int64 address );
//BTT_API    bool                 BttFileMakeFloats(         int file_index, __int64 address, int nQwords );
//BTT_API    bool                 BttFileRunFloat(           int file_index );
//BTT_API    bool                 BttFileRunFloats(          int file_index, int nQwords );
//BTT_API    bool                 BttFileMakeDouble(         int file_index, __int64 address );
//BTT_API    bool                 BttFileMakeDoubles(        int file_index, __int64 address, int nQwords );
//BTT_API    bool                 BttFileRunDouble(          int file_index );
//BTT_API    bool                 BttFileRunDoubles(         int file_index, int nQwords );

//BTT_API    bool                 BttFileStringAsciizCP(     int file_index, string_ascii_cp_t ascii_cp );
//BTT_API    bool                 BttFileMakeStringAsciiz(   int file_index, __int64 address, unsigned long size );
//BTT_API    bool                 BttFileMakeStringUnicode(  int file_index, __int64 address, unsigned long size );
//BTT_API    bool                 BttFileRunStringAsciiz(    int file_index, unsigned long nbytes );
//BTT_API    bool                 BttFileRunStringUnicode(   int file_index, unsigned long nbytes );

BTT_API    bool                 BttFileMakeStruct(          int file_index, __int64 address, const wchar_t *pstruct_name );
//BTT_API    bool                 BttFileRunStruct(           int file_index, const wchar_t *pstruct_name );

//BTT_API    int                  BttFileAddTable(           int file_index, const wchar_t *ptitle, int ncolumns );
//BTT_API    bool                 BttFileAddTableHeaders(    int file_index, int table_index, const wchar_t *phdr1, ... );
//BTT_API    bool                 BttFileAddTableRow(        int file_index, int table_index, const wchar_t *pcol1, ... );
//BTT_API    int                  BttFileAddTableDetails(    int file_index, int table_index, int row, int col );
//BTT_API    bool                 BttFileAddTableDetailsRow( int file_index, int table_index, int details_index, const wchar_t *pcol1, const wchar_t *pcol2 );

////////////////////////////////////////////////////////////
